@@ -68,17 +68,19 @@ __all__ = [
 import numpy as np
 import pandas as pd
 
-from sktime.datatypes._series import convert_dict_Series
-
-from sktime.datatypes._panel import convert_dict_Panel
-
 from sktime.datatypes._check import mtype as infer_mtype
+from sktime.datatypes._hierarchical import convert_dict_Hierarchical
+from sktime.datatypes._panel import convert_dict_Panel
 from sktime.datatypes._registry import mtype_to_scitype
+from sktime.datatypes._series import convert_dict_Series
+from sktime.datatypes._table import convert_dict_Table
 
 # pool convert_dict-s and infer_mtype_dict-s
 convert_dict = dict()
 convert_dict.update(convert_dict_Series)
 convert_dict.update(convert_dict_Panel)
+convert_dict.update(convert_dict_Hierarchical)
+convert_dict.update(convert_dict_Table)
 
 
 def convert(obj, from_type: str, to_type: str, as_scitype: str = None, store=None):
