@@ -11,6 +11,7 @@ from sktime.distances._edr import _EdrDistance
 from sktime.distances._erp import _ErpDistance
 from sktime.distances._euclidean import _EuclideanDistance
 from sktime.distances._lcss import _LcssDistance
+from sktime.distances._msm import _MsmDistance
 from sktime.distances._numba_utils import (
     _compute_pairwise_distance,
     to_numba_pairwise_timeseries,
@@ -26,7 +27,7 @@ from sktime.distances.base import DistanceCallable, MetricInfo, NumbaDistance
 def erp_distance(
     x: np.ndarray,
     y: np.ndarray,
-    window: Union[int, None] = None,
+    window: Union[float, None] = None,
     itakura_max_slope: Union[float, None] = None,
     bounding_matrix: Union[np.ndarray, None] = None,
     g: float = 0.0,
@@ -46,12 +47,12 @@ def erp_distance(
         First time series.
     y: np.ndarray (1d or 2d array)
         Second time series.
-    window: int, defaults = None
-        Integer that is the radius of the sakoe chiba window (if using Sakoe-Chiba
-        lower bounding).
+    window: float, defaults = None
+        Float that is the radius of the sakoe chiba window (if using Sakoe-Chiba
+        lower bounding). Value must be between 0. and 1.
     itakura_max_slope: float, defaults = None
         Gradient of the slope for itakura parallelogram (if using Itakura
-        Parallelogram lower bounding).
+        Parallelogram lower bounding). Value must be between 0. and 1.
     bounding_matrix: np.ndarray (2d of size mxn where m is len(x) and n is len(y)),
                                     defaults = None
         Custom bounding matrix to use. If defined then other lower_bounding params
@@ -71,8 +72,8 @@ def erp_distance(
     Raises
     ------
     ValueError
-        If the sakoe_chiba_window_radius is not an integer.
-        If the itakura_max_slope is not a float or int.
+        If the sakoe_chiba_window_radius is not a float.
+        If the itakura_max_slope is not a float.
         If the value of x or y provided is not a numpy array.
         If the value of x or y has more than 3 dimensions.
         If a metric string provided, and is not a defined valid string.
@@ -114,7 +115,7 @@ def erp_distance(
 def edr_distance(
     x: np.ndarray,
     y: np.ndarray,
-    window: Union[int, None] = None,
+    window: Union[float, None] = None,
     itakura_max_slope: Union[float, None] = None,
     bounding_matrix: Union[np.ndarray, None] = None,
     epsilon: float = None,
@@ -136,12 +137,12 @@ def edr_distance(
         First time series.
     y: np.ndarray (1d or 2d array)
         Second time series.
-    window: int, defaults = None
-        Integer that is the radius of the sakoe chiba window (if using Sakoe-Chiba
-        lower bounding).
+    window: float, defaults = None
+        Float that is the radius of the sakoe chiba window (if using Sakoe-Chiba
+        lower bounding). Value must be between 0. and 1.
     itakura_max_slope: float, defaults = None
         Gradient of the slope for itakura parallelogram (if using Itakura
-        Parallelogram lower bounding).
+        Parallelogram lower bounding). Value must be between 0. and 1.
     bounding_matrix: np.ndarray (2d array), defaults = None
         Custom bounding matrix to use. If defined then other lower_bounding params
         are ignored. The matrix should be structure so that indexes considered in
@@ -164,8 +165,8 @@ def edr_distance(
     Raises
     ------
     ValueError
-        If the sakoe_chiba_window_radius is not an integer.
-        If the itakura_max_slope is not a float or int.
+        If the sakoe_chiba_window_radius is not a float.
+        If the itakura_max_slope is not a float.
         If the value of x or y provided is not a numpy array.
         If the value of x or y has more than 3 dimensions.
         If a metric string provided, and is not a defined valid string.
@@ -208,7 +209,7 @@ def edr_distance(
 def lcss_distance(
     x: np.ndarray,
     y: np.ndarray,
-    window: Union[int, None] = None,
+    window: Union[float, None] = None,
     itakura_max_slope: Union[float, None] = None,
     bounding_matrix: Union[np.ndarray, None] = None,
     epsilon: float = 1.0,
@@ -230,12 +231,12 @@ def lcss_distance(
         First time series.
     y: np.ndarray (1d or 2d array)
         Second time series.
-    window: int, defaults = None
-        Integer that is the radius of the sakoe chiba window (if using Sakoe-Chiba
-        lower bounding).
+    window: float, defaults = None
+        Float that is the radius of the sakoe chiba window (if using Sakoe-Chiba
+        lower bounding). Value must be between 0. and 1.
     itakura_max_slope: float, defaults = None
         Gradient of the slope for itakura parallelogram (if using Itakura
-        Parallelogram lower bounding).
+        Parallelogram lower bounding). Value must be between 0. and 1.
     bounding_matrix: np.ndarray (2d of size mxn where m is len(x) and n is len(y)),
                                     defaults = None
         Custom bounding matrix to use. If defined then other lower_bounding params
@@ -258,8 +259,8 @@ def lcss_distance(
     Raises
     ------
     ValueError
-        If the sakoe_chiba_window_radius is not an integer.
-        If the itakura_max_slope is not a float or int.
+        If the sakoe_chiba_window_radius is not a float.
+        If the itakura_max_slope is not a float.
         If the value of x or y provided is not a numpy array.
         If the value of x or y has more than 2 dimensions.
         If a metric string provided, and is not a defined valid string.
@@ -301,8 +302,8 @@ def lcss_distance(
 def wddtw_distance(
     x: np.ndarray,
     y: np.ndarray,
-    window: Union[int, None] = None,
-    itakura_max_slope: Union[int, None] = None,
+    window: Union[float, None] = None,
+    itakura_max_slope: Union[float, None] = None,
     bounding_matrix: Union[np.ndarray, None] = None,
     compute_derivative: DerivativeCallable = _average_of_slope,
     g: float = 0.0,
@@ -330,12 +331,12 @@ def wddtw_distance(
         First time series.
     y: np.ndarray (1d or 2d array)
         Second time series.
-    window: int, defaults = None
-        Integer that is the radius of the sakoe chiba window (if using Sakoe-Chiba
-        lower bounding).
+    window: float, defaults = None
+        Float that is the radius of the sakoe chiba window (if using Sakoe-Chiba
+        lower bounding). Value must be between 0. and 1.
     itakura_max_slope: float, defaults = None
         Gradient of the slope for itakura parallelogram (if using Itakura
-        Parallelogram lower bounding).
+        Parallelogram lower bounding). Value must be between 0. and 1.
     bounding_matrix: np.ndarray (2d of size mxn where m is len(x) and n is len(y)),
                                     defaults = None
         Custom bounding matrix to use. If defined then other lower_bounding params
@@ -361,8 +362,8 @@ def wddtw_distance(
     Raises
     ------
     ValueError
-        If the sakoe_chiba_window_radius is not an integer.
-        If the itakura_max_slope is not a float or int.
+        If the sakoe_chiba_window_radius is not float.
+        If the itakura_max_slope is not a float.
         If the value of x or y provided is not a numpy array.
         If the value of x or y has more than 2 dimensions.
         If a metric string provided, and is not a defined valid string.
@@ -406,7 +407,7 @@ def wddtw_distance(
 def wdtw_distance(
     x: np.ndarray,
     y: np.ndarray,
-    window: Union[int, None] = None,
+    window: Union[float, None] = None,
     itakura_max_slope: Union[float, None] = None,
     bounding_matrix: np.ndarray = None,
     g: float = 0.0,
@@ -434,12 +435,12 @@ def wdtw_distance(
         First time series.
     y: np.ndarray (1d or 2d array)
         Second time series.
-    window: int, defaults = None
-        Integer that is the radius of the sakoe chiba window (if using Sakoe-Chiba
-        lower bounding).
+    window: float, defaults = None
+        Float that is the radius of the sakoe chiba window (if using Sakoe-Chiba
+        lower bounding). Value must be between 0. and 1.
     itakura_max_slope: float, defaults = None
         Gradient of the slope for itakura parallelogram (if using Itakura
-        Parallelogram lower bounding).
+        Parallelogram lower bounding). Value must be between 0. and 1.
     bounding_matrix: np.ndarray (2d of size mxn where m is len(x) and n is len(y)),
                                     defaults = None
         Custom bounding matrix to use. If defined then other lower_bounding params
@@ -461,8 +462,8 @@ def wdtw_distance(
     Raises
     ------
     ValueError
-        If the sakoe_chiba_window_radius is not an integer.
-        If the itakura_max_slope is not a float or int.
+        If the sakoe_chiba_window_radius is not a float.
+        If the itakura_max_slope is not a float.
         If the value of x or y provided is not a numpy array.
         If the value of x or y has more than 2 dimensions.
         If a metric string provided, and is not a defined valid string.
@@ -503,7 +504,7 @@ def wdtw_distance(
 def ddtw_distance(
     x: np.ndarray,
     y: np.ndarray,
-    window: Union[int, None] = None,
+    window: Union[float, None] = None,
     itakura_max_slope: Union[float, None] = None,
     bounding_matrix: np.ndarray = None,
     compute_derivative: DerivativeCallable = _average_of_slope,
@@ -530,12 +531,12 @@ def ddtw_distance(
         First time series.
     y: np.ndarray (1d or 2d array)
         Second time series.
-    window: int, defaults = None
-        Integer that is the radius of the sakoe chiba window (if using Sakoe-Chiba
-        lower bounding).
+    window: float, defaults = None
+        Float that is the radius of the sakoe chiba window (if using Sakoe-Chiba
+        lower bounding). Value must be between 0. and 1.
     itakura_max_slope: float, defaults = None
         Gradient of the slope for itakura parallelogram (if using Itakura
-        Parallelogram lower bounding).
+        Parallelogram lower bounding). Value must be between 0. and 1.
     bounding_matrix: np.ndarray (2d of size mxn where m is len(x) and n is len(y)),
                                     defaults = None
         Custom bounding matrix to use. If defined then other lower_bounding params
@@ -557,8 +558,8 @@ def ddtw_distance(
     Raises
     ------
     ValueError
-        If the sakoe_chiba_window_radius is not an integer.
-        If the itakura_max_slope is not a float or int.
+        If the sakoe_chiba_window_radius is not a float.
+        If the itakura_max_slope is not a float.
         If the value of x or y provided is not a numpy array.
         If the value of x or y has more than 2 dimensions.
         If a metric string provided, and is not a defined valid string.
@@ -601,14 +602,14 @@ def ddtw_distance(
 def dtw_distance(
     x: np.ndarray,
     y: np.ndarray,
-    window: Union[int, None] = None,
+    window: Union[float, None] = None,
     itakura_max_slope: Union[float, None] = None,
     bounding_matrix: np.ndarray = None,
     **kwargs: Any,
 ) -> float:
     r"""Compute the dynamic time warping (DTW) distance between two time series.
 
-    Originally proposedin [1]_ DTW computes the distance between two time series by
+    Originally proposed in [1]_ DTW computes the distance between two time series by
     considering their alignments during the calculation. This is done by measuring
     the pointwise distance (normally using Euclidean) between all elements of the two
     time series and then using dynamic programming to find the warping path
@@ -625,12 +626,12 @@ def dtw_distance(
         First time series.
     y: np.ndarray (1d or 2d array)
         Second time series.
-    window: int, defaults = None
-        Integer that is the radius of the sakoe chiba window (if using Sakoe-Chiba
-        lower bounding).
+    window: float, defaults = None
+        Float that is the radius of the sakoe chiba window (if using Sakoe-Chiba
+        lower bounding). Value must be between 0. and 1.
     itakura_max_slope: float, defaults = None
         Gradient of the slope for itakura parallelogram (if using Itakura
-        Parallelogram lower bounding).
+        Parallelogram lower bounding). Value must be between 0. and 1.
     bounding_matrix: np.ndarray (2d of size mxn where m is len(x) and n is len(y)),
                                     defaults = None
         Custom bounding matrix to use. If defined then other lower_bounding params
@@ -648,8 +649,8 @@ def dtw_distance(
     Raises
     ------
     ValueError
-        If the sakoe_chiba_window_radius is not an integer.
-        If the itakura_max_slope is not a float or int.
+        If the sakoe_chiba_window_radius is not a float.
+        If the itakura_max_slope is not a float.
         If the value of x or y provided is not a numpy array.
         If the value of x or y has more than 2 dimensions.
         If a metric string provided, and is not a defined valid string.
@@ -687,6 +688,61 @@ def dtw_distance(
     return distance(x, y, metric="dtw", **format_kwargs)
 
 
+def msm_distance(
+    x: np.ndarray,
+    y: np.ndarray,
+    c: float = 0.0,
+    **kwargs: Any,
+) -> float:
+    """Compute the move-split-merge distance.
+
+    This metric uses as building blocks three fundamental operations: Move, Split,
+    and Merge. A Move operation changes the value of a single element, a Split
+    operation converts a single element into two consecutive elements, and a Merge
+    operation merges two consecutive elements into one. Each operation has an
+    associated cost, and the MSM distance between two time series is defined to be
+    the cost of the cheapest sequence of operations that transforms the first time
+    series into the second one.
+
+    Parameters
+    ----------
+    x: np.ndarray (1d or 2d array)
+        First time series.
+    y: np.ndarray (1d or 2d array)
+        Second time series.
+    kwargs: Any
+        Extra kwargs.
+
+    Returns
+    -------
+    float
+        Msm distance between x and y.
+
+    Raises
+    ------
+    ValueError
+        If the value of x or y provided is not a numpy array.
+        If the value of x or y has more than 2 dimensions.
+        If a metric string provided, and is not a defined valid string.
+        If a metric object (instance of class) is provided and doesn't inherit from
+        NumbaDistance.
+        If a resolved metric is not no_python compiled.
+        If the metric type cannot be determined
+
+    References
+    ----------
+    .. [1]A.  Stefan,  V.  Athitsos,  and  G.  Das.   The  Move-Split-Merge  metric
+    for time  series. IEEE  Transactions  on  Knowledge  and  Data  Engineering,
+    25(6):1425–1438, 2013.
+    """
+    format_kwargs = {
+        "c": c,
+    }
+    format_kwargs = {**format_kwargs, **kwargs}
+
+    return distance(x, y, metric="msm", **format_kwargs)
+
+
 def squared_distance(x: np.ndarray, y: np.ndarray, **kwargs: Any) -> float:
     r"""Compute the squared distance between two time series.
 
@@ -816,7 +872,8 @@ def distance(
     metric: str or Callable
         The distance metric to use.
         If a string is given, the value must be one of the following strings:
-        'euclidean', 'squared', 'dtw', 'ddtw', 'wdtw', 'wddtw', 'lcss', 'edr', 'erp'
+        'euclidean', 'squared', 'dtw', 'ddtw', 'wdtw', 'wddtw', 'lcss', 'edr', 'erp',
+        'msm'
 
         If callable then it has to be a distance factory or numba distance callable.
         If you want to pass custom kwargs to the distance at runtime, use a distance
@@ -857,7 +914,7 @@ def distance(
 
     >>> x_2d = np.array([[1, 2, 3, 4], [5, 6, 7, 8]])  # 2d array
     >>> y_2d = np.array([[9, 10, 11, 12], [13, 14, 15, 16]])  # 2d array
-    >>> distance(x_2d, y_2d, metric='dtw', window=3)
+    >>> distance(x_2d, y_2d, metric='dtw', window=0.5)
     512.0
 
     Returns
@@ -941,7 +998,7 @@ def distance_factory(
 
 def pairwise_distance(
     x: np.ndarray,
-    y: np.ndarray,
+    y: np.ndarray = None,
     metric: Union[
         str,
         Callable[
@@ -949,7 +1006,7 @@ def pairwise_distance(
         ],
         Callable[[np.ndarray, np.ndarray], float],
         NumbaDistance,
-    ],
+    ] = "euclidean",
     **kwargs: Any,
 ) -> np.ndarray:
     """Compute the pairwise distance matrix between two time series.
@@ -963,9 +1020,9 @@ def pairwise_distance(
     ----------
     x: np.ndarray (1d, 2d or 3d array)
         First time series.
-    y: np.ndarray (1d, 2d or 3d array)
-        Second time series.
-    metric: str or Callable
+    y: np.ndarray (1d, 2d or 3d array), defaults = None
+        Second time series. If not specified then y is set to the value of x.
+    metric: str or Callable, defaults = 'euclidean'
         The distance metric to use.
         If a string is given, the value must be one of the following strings:
         'euclidean', 'squared', 'dtw', 'ddtw', 'wdtw', 'wddtw', 'lcss', 'edr', 'erp'
@@ -1021,11 +1078,13 @@ def pairwise_distance(
 
     >>> x_2d = np.array([[1, 2, 3, 4], [5, 6, 7, 8]])  # 2d array
     >>> y_2d = np.array([[9, 10, 11, 12], [13, 14, 15, 16]])  # 2d array
-    >>> pairwise_distance(x_2d, y_2d, metric='dtw', window=3)
+    >>> pairwise_distance(x_2d, y_2d, metric='dtw', window=0.5)
     array([[256., 576.],
            [ 58., 256.]])
     """
     _x = to_numba_pairwise_timeseries(x)
+    if y is None:
+        y = x
     _y = to_numba_pairwise_timeseries(y)
     symmetric = np.array_equal(_x, _y)
 
@@ -1088,6 +1147,12 @@ _METRIC_INFOS = [
         dist_func=wddtw_distance,
         dist_instance=_WddtwDistance(),
     ),
+    MetricInfo(
+        canonical_name="msm",
+        aka={"msm", "move-split-merge"},
+        dist_func=msm_distance,
+        dist_instance=_MsmDistance(),
+    ),
 ]
 
 _METRICS = {info.canonical_name: info for info in _METRIC_INFOS}
